@@ -946,9 +946,9 @@ class ip4tables:
         suffix = (last_sort_order, priority)
 
         if postrouting:
-            return ["%%POLICY_SORT_KEY%%", egress + ingress + suffix]
+            return ["%%POLICY_SORT_KEY%%", egress + ingress + suffix + (policy,)]
         else:
-            return ["%%POLICY_SORT_KEY%%", ingress + egress + suffix]
+            return ["%%POLICY_SORT_KEY%%", ingress + egress + suffix + (policy,)]
 
     def build_policy_ingress_egress_pair_rules(
         self,
@@ -1162,9 +1162,12 @@ class ip4tables:
         return rules
 
     def _rule_limit(self, limit):
-        if limit:
-            return ["-m", "limit", "--limit", limit.value]
-        return []
+        if not limit:
+            return []
+        s = ["-m", "limit", "--limit", limit.value]
+        if limit.burst is not None:
+            s += ["--limit-burst", str(limit.burst)]
+        return s
 
     def _rich_rule_chain_suffix(self, rich_rule):
         if type(rich_rule.element) in [
@@ -1610,33 +1613,28 @@ class ip4tables:
         if toport and toport != "":
             to += ":%s" % portStr(toport, "-")
 
+        rules = []
         rule_fragment = []
+        port_fragment = ["-p", protocol, "--dport", portStr(port)]
         if rich_rule:
             chain_suffix = self._rich_rule_chain_suffix(rich_rule)
             rule_fragment = self._rich_rule_priority_fragment(rich_rule)
             rule_fragment += self._rich_rule_destination_fragment(rich_rule.destination)
             rule_fragment += self._rich_rule_source_fragment(rich_rule.source)
+
+            rules.append(
+                self._rich_rule_log(
+                    policy, rich_rule, enable, "nat", rule_fragment + port_fragment
+                )
+            )
         else:
             chain_suffix = "allow"
 
-        rules = []
-        if rich_rule:
-            rules.append(
-                self._rich_rule_log(policy, rich_rule, enable, "nat", rule_fragment)
-            )
         rules.append(
             ["-t", "nat", add_del, "%s_%s" % (_policy, chain_suffix)]
             + rule_fragment
-            + [
-                "-p",
-                protocol,
-                "--dport",
-                portStr(port),
-                "-j",
-                "DNAT",
-                "--to-destination",
-                to,
-            ]
+            + port_fragment
+            + ["-j", "DNAT", "--to-destination", to]
         )
 
         return rules
@@ -1784,16 +1782,19 @@ class ip6tables(ip4tables):
 
     def build_rpfilter_rules(self, log_denied=False):
         rules = []
+        rpfilter_fragment = ["-m", "rpfilter", "--invert", "--validmark"]
+        if self._fw._ipv6_rpfilter == "loose":
+            rpfilter_fragment += ["--loose"]
+
         rules.append(
             [
                 "-I",
                 "PREROUTING",
                 "-t",
                 "mangle",
-                "-m",
-                "rpfilter",
-                "--invert",
-                "--validmark",
+            ]
+            + rpfilter_fragment
+            + [
                 "-j",
                 "DROP",
             ]
@@ -1805,10 +1806,9 @@ class ip6tables(ip4tables):
                     "PREROUTING",
                     "-t",
                     "mangle",
-                    "-m",
-                    "rpfilter",
-                    "--invert",
-                    "--validmark",
+                ]
+                + rpfilter_fragment
+                + [
                     "-j",
                     "LOG",
                     "--log-prefix",
